@@ -19,7 +19,8 @@ use \OCP\IContainer;
 use \OCA\Bookmarks\Controller\WebViewController;
 use OCA\Bookmarks\Controller\Rest\TagsController;
 use OCA\Bookmarks\Controller\Rest\BookmarkController;
-use OCA\Bookmarks\Controller\Rest\PublicController;
+use OCA\Bookmarks\Controller\Rest\InternalTagsController;
+use OCA\Bookmarks\Controller\Rest\InternalBookmarkController;
 use OCP\IUser;
 
 class Application extends App {
@@ -54,9 +55,23 @@ class Application extends App {
 				$c->query('AppName'),
 				$c->query('Request'),
 				$c->query('ServerContainer')->getUserSession()->getUser()->getUID(),
-				$c->query('ServerContainer')->getDb(),
+				$c->query('ServerContainer')->getDatabaseConnection(),
 				$c->query('ServerContainer')->getL10NFactory()->get('bookmarks'),
-				$c->query('ServerContainer')->query(Bookmarks::class)
+				$c->query('ServerContainer')->query(Bookmarks::class),
+				$c->query('ServerContainer')->getUserManager()
+			);
+		});
+		
+		$container->registerService('InternalBookmarkController', function($c) {
+			/** @var IContainer $c */
+			return new InternalBookmarkController(
+				$c->query('AppName'),
+				$c->query('Request'),
+				$c->query('ServerContainer')->getUserSession()->getUser()->getUID(),
+				$c->query('ServerContainer')->getDatabaseConnection(),
+				$c->query('ServerContainer')->getL10NFactory()->get('bookmarks'),
+				$c->query('ServerContainer')->query(Bookmarks::class),
+				$c->query('ServerContainer')->getUserManager()
 			);
 		});
 
@@ -69,17 +84,16 @@ class Application extends App {
 				$c->query('ServerContainer')->query(Bookmarks::class)
 			);
 		});
-
-		$container->registerService('PublicController', function($c) {
+		
+		$container->registerService('InternalTagsController', function($c) {
 			/** @var IContainer $c */
-			return new PublicController(
+			return new InternalTagsController(
 				$c->query('AppName'),
 				$c->query('Request'),
-				$c->query('ServerContainer')->query(Bookmarks::class),
-				$c->query('ServerContainer')->getUserManager()
+				$c->query('ServerContainer')->getUserSession()->getUser()->getUID(),
+				$c->query('ServerContainer')->query(Bookmarks::class)
 			);
 		});
-
 	}
 
 }
